"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const classnames_1 = require("classnames");
const Splitbar_1 = require("./Splitbar");
const PX_REG = new RegExp('\\d+(px)$');
const PERCENT_REG = new RegExp('\\d+(%)$');
const NUM_REG = new RegExp('\\d+\\.?\\d*');
const Splitter = (props) => {
    const wrapperRef = React.useRef(null);
    const itemsRef = React.useRef([]);
    const [isResizing, setIsResizing] = React.useState(false);
    const [itemSizeList, setItemSizeList] = React.useState([]);
    React.useEffect(() => {
        var _a, _b, _c, _d;
        let noSizeCount = 0;
        let _itemSizes = [];
        const barsize = (props.items.length - 1) * (((_a = props.splitbar) === null || _a === void 0 ? void 0 : _a.size) || 1);
        const wrapperSize = (props.direction === "vertical" ? (((_b = wrapperRef.current) === null || _b === void 0 ? void 0 : _b.clientHeight) || 0) : (((_c = wrapperRef.current) === null || _c === void 0 ? void 0 : _c.clientWidth) || 0)) - barsize;
        props.items.forEach((item, index) => {
            const size = calcSize(item.defaultSize || 0, wrapperSize);
            const minSize = calcSize(item.minSize || 0, wrapperSize);
            const maxSize = calcSize(item.maxSize || 0, wrapperSize);
            _itemSizes[index] = {
                px: size,
                minPx: minSize,
                maxPx: maxSize,
                percent: size / wrapperSize,
                minPercent: minSize / wrapperSize,
                maxPercent: maxSize / wrapperSize
            };
            noSizeCount += size > 0 ? 0 : 1;
        });
        const pxSum = _itemSizes.reduce((acc, cur) => acc + cur.px, 0);
        const restSize = wrapperSize - pxSum;
        _itemSizes = _itemSizes.map(item => {
            if (item.px > 0 && item.percent > 0) {
                return item;
            }
            const px = restSize / noSizeCount;
            return Object.assign(Object.assign({}, item), { px, percent: px / wrapperSize });
        });
        setItemSizeList(_itemSizes);
        handleItemsStyle(_itemSizes);
        (_d = props.onResize) === null || _d === void 0 ? void 0 : _d.call(props, _itemSizes.map(m => m.px), _itemSizes.map(m => m.percent));
    }, [props.direction, wrapperRef.current]);
    React.useEffect(() => {
        const observer = new ResizeObserver((e) => {
            var _a, _b, _c, _d;
            const barsize = (props.items.length - 1) * (((_a = props.splitbar) === null || _a === void 0 ? void 0 : _a.size) || 1);
            const wrapperSize = (props.direction === "vertical" ? (((_b = e[0].target) === null || _b === void 0 ? void 0 : _b.clientHeight) || 0) : (((_c = e[0].target) === null || _c === void 0 ? void 0 : _c.clientWidth) || 0)) - barsize;
            itemSizeList.forEach((item) => {
                item.px = item.percent * wrapperSize;
            });
            (_d = props.onResize) === null || _d === void 0 ? void 0 : _d.call(props, itemSizeList.map(m => m.px), itemSizeList.map(m => m.percent));
        });
        if (wrapperRef === null || wrapperRef === void 0 ? void 0 : wrapperRef.current) {
            observer.observe(wrapperRef === null || wrapperRef === void 0 ? void 0 : wrapperRef.current);
        }
        return () => {
            observer.disconnect();
        };
    }, [itemSizeList, props.direction, wrapperRef === null || wrapperRef === void 0 ? void 0 : wrapperRef.current]);
    const calcSize = React.useCallback((size, wrapperSize = 0) => {
        var _a, _b;
        if (typeof size === 'number') {
            return Math.abs(size);
        }
        if (typeof size === 'string') {
            if (PX_REG.test(size)) {
                return Math.abs(parseFloat(((_a = size.match(NUM_REG)) === null || _a === void 0 ? void 0 : _a[0]) || '0'));
            }
            if (PERCENT_REG.test(size)) {
                return Math.abs(parseFloat(((_b = size.match(NUM_REG)) === null || _b === void 0 ? void 0 : _b[0]) || '0') / 100) * wrapperSize;
            }
        }
        return 0;
    }, [wrapperRef.current]);
    const handleItemsStyle = (inItemSizeList) => {
        inItemSizeList.forEach((item, index) => {
            var _a, _b;
            if (props.direction === 'vertical') {
                const _style = [`height: ${(item.percent || 0) * 100}%`];
                if (props.items[index].maxSize) {
                    _style.push(`max-height: ${props.items[index].maxSize}`);
                }
                if (props.items[index].minSize) {
                    _style.push(`min-height: ${props.items[index].minSize}`);
                }
                (_a = itemsRef.current[index]) === null || _a === void 0 ? void 0 : _a.setAttribute('style', _style.join(';'));
            }
            else {
                const _style = [`width: ${(item.percent || 0) * 100}%`];
                if (props.items[index].maxSize) {
                    _style.push(`max-width: ${props.items[index].maxSize}`);
                }
                if (props.items[index].minSize) {
                    _style.push(`min-width: ${props.items[index].minSize}`);
                }
                (_b = itemsRef.current[index]) === null || _b === void 0 ? void 0 : _b.setAttribute('style', _style.join(';'));
            }
        });
    };
    const handleMouseDown = (e, index) => {
        var _a, _b, _c;
        e.persist();
        const px1 = itemSizeList[index].px;
        const px2 = itemSizeList[index + 1].px;
        const item1 = itemSizeList[index];
        const item2 = itemSizeList[index + 1];
        const barsize = (props.items.length - 1) * (((_a = props.splitbar) === null || _a === void 0 ? void 0 : _a.size) || 1);
        const wrapperSize = (props.direction === "vertical" ? (((_b = wrapperRef.current) === null || _b === void 0 ? void 0 : _b.clientHeight) || 0) : (((_c = wrapperRef.current) === null || _c === void 0 ? void 0 : _c.clientWidth) || 0)) - barsize;
        const calcFunc = (offset) => {
            var _a;
            const _px1 = px1 + offset;
            const _px2 = px2 - offset;
            const isMin1 = _px1 <= item1.minPx && item1.minPx > 0;
            const isMin2 = _px2 <= item2.minPx && item2.minPx > 0;
            const isMax1 = _px1 >= item1.maxPx && item1.maxPx > 0;
            const isMax2 = _px2 >= item2.maxPx && item2.maxPx > 0;
            if (_px1 >= 0 && _px2 >= 0 && !isMin1 && !isMin2 && !isMax1 && !isMax2) {
                item1.px = _px1;
                item1.percent = _px1 / wrapperSize;
                item2.px = _px2;
                item2.percent = _px2 / wrapperSize;
                setItemSizeList([...itemSizeList]);
                handleItemsStyle(itemSizeList);
                (_a = props.onResize) === null || _a === void 0 ? void 0 : _a.call(props, itemSizeList.map(m => m.px), itemSizeList.map(m => m.percent));
            }
        };
        const onMouseMove = (mEvent) => {
            mEvent.preventDefault();
            mEvent.stopPropagation();
            if (props.direction !== 'vertical') {
                const offsetX = mEvent.clientX - e.clientX;
                calcFunc(offsetX);
            }
            if (props.direction === 'vertical') {
                const offsetY = mEvent.clientY - e.clientY;
                calcFunc(offsetY);
            }
            setIsResizing(true);
        };
        const onMouseUp = () => {
            setIsResizing(false);
            document === null || document === void 0 ? void 0 : document.removeEventListener('mouseup', onMouseUp, false);
            document === null || document === void 0 ? void 0 : document.removeEventListener('mousemove', onMouseMove, false);
        };
        document === null || document === void 0 ? void 0 : document.addEventListener('mouseup', onMouseUp, false);
        document === null || document === void 0 ? void 0 : document.addEventListener('mousemove', onMouseMove, false);
    };
    const classes = (0, classnames_1.default)('ihc-splitter-wrapper', {
        'ihc-splitter-bordered': props.bordered,
        'ihc-splitter-vertical': props.direction === 'vertical',
        'ihc-splitter-horizontal': props.direction === 'horizontal',
        'ihc-splitter-resizing': isResizing,
    }, props.className);
    return (React.createElement("div", { ref: wrapperRef, className: classes, style: props.style }, props.items.map((item, index) => {
        return (React.createElement(React.Fragment, { key: `splititem_${item.key || index}` },
            React.createElement("div", { key: `splititem_${item.key || index}`, className: (0, classnames_1.default)("ihc-splitter-item"), ref: ref => itemsRef.current[index] = ref }, item.content),
            index < props.items.length - 1 && React.createElement(Splitbar_1.default, Object.assign({}, props.splitbar, { key: `splitbar_${item.key || index}`, direction: props.direction, onMouseDown: (e) => handleMouseDown(e, index) }))));
    })));
};
exports.default = Splitter;
